/*_|_|  ADDGUIQB.C  FREEWARE RESOURCE source code module
_|_|_|  640x480x16 mode 12h graphical user interface for QB45
_|_|_|  First release 01/01/2005  by  Kurt Kuzba and
_|_|_|  Alternate Dementians Development.
_|_|_|  Features:
_|_|_|    Mouse interface and custom mouse cursors
_|_|_|    Bit blit and transparent bit blit
_|_|_|    Onboard character set (8x12) for 80x40 text screen
_|_|_|    Text output and text over graphic for native QB strings
_|_|_|    150 onboard button control definitions allowed
_|_|_|    Automatic tracking of registered button control objects
_|_|_|    Built in graphics primitives set
_|_|_|    Default graphical mouse pointer definition
_|_|_|    Support for radio, menu, and text buttons
_|_|_|    Seven color themes supported in code for buttons and frames
_|_|*/


#include <conio.h>
#include <dos.h>
#include "mouse.h"
#define DOS_INT 0x21
#define MOUSE(workregs) int86(MSMOUSE, &workregs, &workregs)
#define MOUSEX(workregs, sregs) int86x(DOS_INT, &workregs, &workregs, &sregs)
#define NULL 0

/*=====================================================================*/
/**/

typedef struct {
	int wide, high, v_pos, h_pos, l_button, r_button, sweetleft, sweetdown;
	char pic[1024];
} mousedat;

static mousedat Md, Mm;

/*=====================================================================*/
/**/

typedef struct {
	int type, left, right, top, bottom, colorfore, colorback, len;
	char text[32];
} guibutton;

static guibutton BtnDat[151];

/*=====================================================================*/
/**/

typedef struct {
	int len;
	char *text;
} qbstring;

/*=====================================================================*/
/**/

static int bgcolor = 0;
static int mid = 7, hil = 15, lol = 8;

/*=====================================================================*/
/**/

int ms_reset(int *mousetype)
{
	union REGS workregs;
	struct SREGS sregs;

	/* check the vector     */

	segread (&sregs);
	workregs.h.ah = 0x35;     /* DOS get vector */
	workregs.h.al = 0x33;     /* mouse vector   */
	intdosx(&workregs, &workregs, &sregs);

	/* ES:BX now contains the pointer to the interrupt handler */

	if (sregs.es == 0 && workregs.x.bx == 0)
				return 0;

	workregs.x.ax = 0;
	MOUSE(workregs);
	*mousetype = workregs.x.bx;
	return workregs.x.ax;
}

/*=====================================================================*/
/**/

int ms_get_mouse_pos(int *horizpos, int *vertpos)
{
	union REGS workregs;
	workregs.x.ax = 3;
	MOUSE(workregs);
	*horizpos = workregs.x.cx;
	*vertpos  = workregs.x.dx;
	return(workregs.x.bx);
}

/*=====================================================================*/
/**/

void ms_set_mouse_pos(int horizpos, int vertpos)
{
	union REGS workregs;
	workregs.x.ax = 4;
	workregs.x.cx = horizpos;
	workregs.x.dx = vertpos;
	MOUSE(workregs);
}

/*=====================================================================*/
/**/

static int pix[8] = { 128,  64,  32,  16,   8,   4,   2,   1 };
static int pox[8] = { 127, 191, 223, 239, 247, 251, 253, 254 };

void pascal putpixel(int *y, int *x, int *color)
{
	unsigned char far *V = (unsigned char far*)0xa0000000l;
	int bit = pix[*y & 7], bitmask = pox[*y & 7];
	V += ((*x * 80) + *y / 8);
	if(*y < 0 || *x < 0 || *y > 639 || *x > 479)
		return;

	_asm {
		mov dx, 3ceh
		mov al, 04h
		out dx, al
		mov dx, 3c4h
		mov al, 02h
		out dx, al
		mov dx, 3cfh
		mov al, 00h
		out dx, al
		mov dx, 3c5h
		mov al, 01h
		out dx, al
	}

	if(*color & 1)
	{
		*V |= bit;
	}
	else
	{
		*V &= bitmask;
	}

	_asm {
		mov dx, 3cfh
		mov al, 01h
		out dx, al
		mov dx, 3c5h
		mov al, 02h
		out dx, al
	}

	if(*color & 2)
	{
		*V |= bit;
	}
	else
	{
		*V &= bitmask;
	}

	_asm {
		mov dx, 3cfh
		mov al, 02h
		out dx, al
		 mov dx, 3c5h
		mov al, 04h
		out dx, al
	}

	if(*color & 4)
	{
		*V |= bit;
	}
	else
	{
		*V &= bitmask;
	}

	_asm {
		mov dx, 3cfh
		mov al, 03h
		out dx, al
		mov dx, 3c5h
		mov al, 08h
		out dx, al
	}

	if(*color & 8)
	{
		*V |= bit;
	}
	else
	{
		*V &= bitmask;
	}
}

/*=====================================================================*/
/**/

int pascal getpixel(int *y, int *x)
{
	unsigned char far *V = (unsigned char far*)0xa0000000l;
	int bit = pix[*y & 7], bitmask = pox[*y & 7], color = 0;
	V += ((*x * 80) + *y / 8);
	if(*y < 0 || *x < 0 || *y > 639 || *x > 479)
		return 0;

	_asm {
		mov dx, 3ceh
		mov al, 04h
		out dx, al
		mov dx, 3c4h
		mov al, 02h
		out dx, al
		mov dx, 3cfh
		mov al, 00h
		out dx, al
		mov dx, 3c5h
		mov al, 01h
		out dx, al
	}

	if(*V & bit)
		color |= 1;

	_asm {
		mov dx, 3cfh
		mov al, 01h
		out dx, al
		mov dx, 3c5h
		mov al, 02h
		out dx, al
	}

	if(*V & bit)
		color |= 2;

	_asm {
		mov dx, 3cfh
		mov al, 02h
		out dx, al
		mov dx, 3c5h
		mov al, 04h
		out dx, al
	}

	if(*V & bit)
		color |= 4;

	_asm {
		mov dx, 3cfh
		mov al, 03h
		out dx, al
		mov dx, 3c5h
		mov al, 08h
		out dx, al
	}

	if(*V & bit)
		color |= 8;

	return color;
}

/*=====================================================================*/
/**/

int getdot(int y, int x)
{
	return getpixel(&y, &x);
}

/*=====================================================================*/
/**/

void pascal fourplanes(int y, int x, int color, int plane)
{
	unsigned char far *V = (unsigned char far*)0xa0000000l;
	int bit = pix[y & 7], bitmask = pox[y & 7];
	V += ((x * 80) + y / 8);
	if(y < 0 || x < 0 || y > 639 || x > 479)
		return;

	if(color & plane)
	{
		*V |= bit;
	}
	else
	{
		*V &= bitmask;
	}
}

void pascal fourplanebyte(int y, int x, int color, int plane)
{
	unsigned char far *V = (unsigned char far*)0xa0000000l;
	V += ((x * 80) + y / 8);
	if(y < 0 || x < 0 || y > 639 || x > 479)
		return;

	if(color & plane)
	{
		*V = 255;
	}
	else
	{
		*V = 0;
	}
}

void horizontalline(int y, int x, int x2, int c)
{
	int left, right, horz;
	if(x < x2)
	{
		left = x;
		right = x2;
	}
	else
	{
		left = x2;
		right = x;
	}

	_asm {
		mov dx, 3ceh
		mov al, 04h
		out dx, al
		mov dx, 3c4h
		mov al, 02h
		out dx, al
		mov dx, 3cfh
		mov al, 00h
		out dx, al
		mov dx, 3c5h
		mov al, 01h
		out dx, al
	}

	horz = left;
	do
	{
		if(horz & 7 == 0 && horz + 7 <= right)
		{
			fourplanebyte(horz, y, c, 1);
			horz += 8;
		}
		else
		{
			fourplanes(horz, y, c, 1);
			horz ++;
		}
	} while(horz <= right);

	_asm {
		mov dx, 3cfh
		mov al, 01h
		out dx, al
		mov dx, 3c5h
		mov al, 02h
		out dx, al
	}

	horz = left;
	do
	{
		if(horz & 7 == 0 && horz + 7 <= right)
		{
			fourplanebyte(horz, y, c, 2);
			horz += 8;
		}
		else
		{
			fourplanes(horz, y, c, 2);
			horz ++;
		}
	} while(horz <= right);

	_asm {
		mov dx, 3cfh
		mov al, 02h
		out dx, al
		 mov dx, 3c5h
		mov al, 04h
		out dx, al
	}

	horz = left;
	do
	{
		if(horz & 7 == 0 && horz + 7 <= right)
		{
			fourplanebyte(horz, y, c, 4);
			horz += 8;
		}
		else
		{
			fourplanes(horz, y, c, 4);
			horz ++;
		}
	} while(horz <= right);

	_asm {
		mov dx, 3cfh
		mov al, 03h
		out dx, al
		mov dx, 3c5h
		mov al, 08h
		out dx, al
	}

	horz = left;
	do
	{
		if(horz & 7 == 0 && horz + 7 <= right)
		{
			fourplanebyte(horz, y, c, 8);
			horz += 8;
		}
		else
		{
			fourplanes(horz, y, c, 8);
			horz ++;
		}
	} while(horz <= right);
}

void verticalline(int x, int y, int y2, int c)
{
	int top, bottom, vert;
	if(y < y2)
	{
		top = y;
		bottom = y2;
	}
	else
	{
		top = y2;
		bottom = y;
	}

	_asm {
		mov dx, 3ceh
		mov al, 04h
		out dx, al
		mov dx, 3c4h
		mov al, 02h
		out dx, al
		mov dx, 3cfh
		mov al, 00h
		out dx, al
		mov dx, 3c5h
		mov al, 01h
		out dx, al
	}

	for(vert = top ; vert <= bottom ; vert++)
		fourplanes(x, vert, c, 1);

	_asm {
		mov dx, 3cfh
		mov al, 01h
		out dx, al
		mov dx, 3c5h
		mov al, 02h
		out dx, al
	}

	for(vert = top ; vert <= bottom ; vert++)
		fourplanes(x, vert, c, 2);

	_asm {
		mov dx, 3cfh
		mov al, 02h
		out dx, al
		 mov dx, 3c5h
		mov al, 04h
		out dx, al
	}

	for(vert = top ; vert <= bottom ; vert++)
		fourplanes(x, vert, c, 4);

	_asm {
		mov dx, 3cfh
		mov al, 03h
		out dx, al
		mov dx, 3c5h
		mov al, 08h
		out dx, al
	}

	for(vert = top ; vert <= bottom ; vert++)
		fourplanes(x, vert, c, 8);
}

void pascal cbline(int *x, int *y, int *x2, int *y2, int *c)
{
	int i, steep, sx, sy, dx, dy, e, ex, ey, ix, iy, plane;

	if (*x == *x2)
	{
		verticalline(*x, *y, *y2, *c);
		return;
	}

	if (*y == *y2)
	{
		horizontalline(*y, *x, *x2, *c);
		return;
	}

	_asm {
		mov dx, 3ceh
		mov al, 04h
		out dx, al
		mov dx, 3c4h
		mov al, 02h
		out dx, al
		mov dx, 3cfh
		mov al, 00h
		out dx, al
		mov dx, 3c5h
		mov al, 01h
		out dx, al
	}

	steep = 0;
	ix = *x;
	iy = *y;

	dx = ((*x2 - *x) > 0)
		?   *x2 - *x
		: -(*x2 - *x);
	sx = ((*x2 - *x) > 0)
		?  1
		: -1;
	dy = ((*y2 - *y) > 0)
		?   *y2 - *y
		: -(*y2 - *y);
	sy = ((*y2 - *y) > 0)
		?  1
		: -1;

	if(dy > dx)
	{
		iy = *x;  ix =  *y;               /* swap  x and  y */
		steep = dx; dx = dy; dy = steep;  /* swap dx and dy */
		steep = sx; sx = sy; sy = steep;  /* swap sx and sy */
		steep = 1;
	}

	ex = dx << 1;
	ey = dy << 1;
	e = ey - dx;

	for(i = 0 ; i < dx ; i++)
	{
		if(steep)
		{
			fourplanes(iy, ix, *c, 1);
		}
		else
		{
			fourplanes(ix, iy, *c, 1);
		}
		while(e >= 0)
		{
			iy += sy;
			e -= ex;
		}
		ix += sx;
		e += ey;
	}
	fourplanes(*x2, *y2, *c, 1);

	_asm {
		mov dx, 3cfh
		mov al, 01h
		out dx, al
		mov dx, 3c5h
		mov al, 02h
		out dx, al
	}

	steep = 0;
	ix = *x;
	iy = *y;

	dx = ((*x2 - *x) > 0)
		?   *x2 - *x
		: -(*x2 - *x);
	sx = ((*x2 - *x) > 0)
		?  1
		: -1;
	dy = ((*y2 - *y) > 0)
		?   *y2 - *y
		: -(*y2 - *y);
	sy = ((*y2 - *y) > 0)
		?  1
		: -1;

	if(dy > dx)
	{
		iy = *x;  ix =  *y;               /* swap  x and  y */
		steep = dx; dx = dy; dy = steep;  /* swap dx and dy */
		steep = sx; sx = sy; sy = steep;  /* swap sx and sy */
		steep = 1;
	}

	ex = dx << 1;
	ey = dy << 1;
	e = ey - dx;
	for(i = 0 ; i < dx ; i++)
	{
		if(steep)
		{
			fourplanes(iy, ix, *c, 2);
		}
		else
		{
			fourplanes(ix, iy, *c, 2);
		}
		while(e >= 0)
		{
			iy += sy;
			e -= ex;
		}
		ix += sx;
		e += ey;
	}
	fourplanes(*x2, *y2, *c, 2);

	_asm {
		mov dx, 3cfh
		mov al, 02h
		out dx, al
		 mov dx, 3c5h
		mov al, 04h
		out dx, al
	}

	steep = 0;
	ix = *x;
	iy = *y;

	dx = ((*x2 - *x) > 0)
		?   *x2 - *x
		: -(*x2 - *x);
	sx = ((*x2 - *x) > 0)
		?  1
		: -1;
	dy = ((*y2 - *y) > 0)
		?   *y2 - *y
		: -(*y2 - *y);
	sy = ((*y2 - *y) > 0)
		?  1
		: -1;

	if(dy > dx)
	{
		iy = *x;  ix =  *y;               /* swap  x and  y */
		steep = dx; dx = dy; dy = steep;  /* swap dx and dy */
		steep = sx; sx = sy; sy = steep;  /* swap sx and sy */
		steep = 1;
	}

	ex = dx << 1;
	ey = dy << 1;
	e = ey - dx;

	for(i = 0 ; i < dx ; i++)
	{
		if(steep)
		{
			fourplanes(iy, ix, *c, 4);
		}
		else
		{
			fourplanes(ix, iy, *c, 4);
		}
		while(e >= 0)
		{
			iy += sy;
			e -= ex;
		}
		ix += sx;
		e += ey;
	}
	fourplanes(*x2, *y2, *c, 4);

	_asm {
		mov dx, 3cfh
		mov al, 03h
		out dx, al
		mov dx, 3c5h
		mov al, 08h
		out dx, al
	}

	steep = 0;
	ix = *x;
	iy = *y;

	dx = ((*x2 - *x) > 0)
		?   *x2 - *x
		: -(*x2 - *x);
	sx = ((*x2 - *x) > 0)
		?  1
		: -1;
	dy = ((*y2 - *y) > 0)
		?   *y2 - *y
		: -(*y2 - *y);
	sy = ((*y2 - *y) > 0)
		?  1
		: -1;

	if(dy > dx)
	{
		iy = *x;  ix =  *y;               /* swap  x and  y */
		steep = dx; dx = dy; dy = steep;  /* swap dx and dy */
		steep = sx; sx = sy; sy = steep;  /* swap sx and sy */
		steep = 1;
	}

	ex = dx << 1;
	ey = dy << 1;
	e = ey - dx;
	for(i = 0 ; i < dx ; i++)
	{
		if(steep)
		{
			fourplanes(iy, ix, *c, 8);
		}
		else
		{
			fourplanes(ix, iy, *c, 8);
		}
		while(e >= 0)
		{
			iy += sy;
			e -= ex;
		}
		ix += sx;
		e += ey;
	}
	fourplanes(*x2, *y2, *c, 8);
}

/*=====================================================================*/
/**/

void pascal clearscreen(int *c)
{
	unsigned int x, xc;
	unsigned char far *v = (unsigned char far*)0xa0000000l;
	bgcolor = *c;

	_asm {
		mov dx, 3ceh
		mov al, 04h
		out dx, al
		mov dx, 3c4h
		mov al, 02h
		out dx, al
		mov dx, 3cfh
		mov al, 00h
		out dx, al
		mov dx, 3c5h
		mov al, 01h
		out dx, al
	}

	xc = (*c & 1) ? 255 : 0;
	for(x = 0 ; x < 38400 ; x++)
		*(v + x) = (char)xc;

	_asm {
		mov dx, 3cfh
		mov al, 01h
		out dx, al
		mov dx, 3c5h
		mov al, 02h
		out dx, al
	}

	xc = (*c & 2) ? 255 : 0;
	for(x = 0 ; x < 38400 ; x++)
		*(v + x) = (char)xc;

	_asm {
		mov dx, 3cfh
		mov al, 02h
		out dx, al
		mov dx, 3c5h
		mov al, 04h
		out dx, al
	}

	xc = (*c & 4) ? 255 : 0;
	for(x = 0 ; x < 38400 ; x++)
		*(v + x) = (char)xc;

	_asm {
		mov dx, 3cfh
		mov al, 03h
		out dx, al
		mov dx, 3c5h
		mov al, 08h
		out dx, al
	}

	xc = (*c & 8) ? 255 : 0;
	for(x = 0 ; x < 38400 ; x++)
		*(v + x) = (char)xc;
}

/*=====================================================================*/
/**/

void pascal cbrect(int *x, int *y, int *x2, int *y2, int *c)
{
	cbline(x  , y  , x2 , y  , c);
	cbline(x  , y2 , x2 , y2 , c);
	cbline(x  , y  , x  , y2 , c);
	cbline(x2 , y  , x2 , y2 , c);
}

/*=====================================================================*/
/**/

void pascal framedown(int *x, int *y, int *x2, int *y2)
{
	int xx = *x + 1, xx2 = *x2 - 1, yy = *y + 1, yy2 = *y2 - 1;
	cbrect(x    , y    , x2   , y2   , &mid);
	cbline(&xx2 , &yy  , &xx2 , &yy2 , &hil);
	cbline(&xx  , &yy2 , &xx2 , &yy2 , &hil);
	cbline(&xx  , &yy  , &xx2 , &yy  , &lol);
	cbline(&xx  , &yy  , &xx  , &yy2 , &lol);
}

/*=====================================================================*/
/**/

void pascal frameup(int *x, int *y, int *x2, int *y2)
{
	int xx = *x + 1, xx2 = *x2 - 1, yy = *y + 1, yy2 = *y2 - 1;
	cbrect(&xx , &yy , &xx2 , &yy2 , &mid);
	cbline(x   , y2  , x2   , y2   , &lol);
	cbline(x   , y   , x2   , y    , &hil);
	cbline(x   , y   , x    , y2   , &hil);
	cbline(x2  , y   , x2   , y2   , &lol);
}

/*=====================================================================*/
/**/

void pascal cbsquare(int *x, int *y, int *l, int *c)
{
	int x2 = *x + *l, y2 = *y + *l;
	cbrect(x, y, &x2, &y2, c);
}

/*=====================================================================*/
/**/

void pascal cbblock(int *x, int *y, int *x2, int *y2, int *c)
{
	int iX, iY;
	for(iY = *y ; iY <= *y2 ; iY++)
		cbline(x, &iY, x2, &iY, c);
}

/*=====================================================================*/
/**/

void pascal cbsqrblock(int *x, int *y, int *l, int *c)
{
	int iX = *x + *l, iY;
	for(iY = *y + *l ; iY >= *y ; iY--)
		cbline(x, &iY, &iX, &iY, c);
}

/*=====================================================================*/
/**/

void pascal cboval(int *xc, int *yc, int *r, int *c, int *aw, int *ah)
{
	int x = 0, d = (1 - *r) << 1, w, cr = *r, xl, yl;

	if(*aw > *ah)
	{
		w = (*aw / *ah) << 1;

		while(cr >= 0)
		{
			xl = *xc + x;
			yl = *yc + cr;
			putpixel(&xl, &yl, c);
			yl = *yc - cr;
			putpixel(&xl, &yl, c);
			xl = *xc - x;
			putpixel(&xl, &yl, c);
			yl = *yc + cr;
			putpixel(&xl, &yl, c);

			if(d + cr > 0)
				d -= (w * --cr) - 1;

			if(x > d)
				d += ((++x) << 1) + 1;
		}

	}
	else
	{
		w = (*ah / *aw) << 1;

		while(cr >= 0)
		{
			xl = *xc + cr;
			yl = *yc + x;
			putpixel(&xl, &yl, c);
			yl = *yc - x;
			putpixel(&xl, &yl, c);
			xl = *xc - cr;
			putpixel(&xl, &yl, c);
			yl = *yc + x;
			putpixel(&xl, &yl, c);

			if(d + cr > 0)
				d -= (w * --cr) - 1;

			if(x > d)
				d += ((++x) << 1) + 1;
		}

	}
}

/*=====================================================================*/
/**/

void pascal cbcircle(int *xc, int *yc, int *r, int *c)
{
	int w = 4, h = 3;
	cboval(xc, yc, r, c, &w, &h);
}

/*=====================================================================*/
/**/

void pascal cbovaldisc(int *xc, int *yc, int *r, int *c, int *hg, int *wd)
{
	int x = 0, d = (1 - *r) << 1, cr = *r, w, xl, ya, yb;
	if(*wd > *hg)
	{
		w = (*wd / *hg) << 1;

		while(cr >= 0)
		{
			xl = *xc + x ;
			ya = *yc + cr;
			yb = *yc - cr;
			cbline(&xl, &ya, &xl, &yb, c);
			xl = *xc - x;
			cbline(&xl, &ya, &xl, &yb, c);

			if(d + cr > 0)
				d -= (w * --cr) - 1;

			if(x > d)
				d += ((++x) << 1) + 1;
		}

	}
	else
	{
		w = (*hg / *wd) << 1;

		while(cr >= 0)
		{
			xl = *xc + cr;
			ya = *yc + x ;
			yb = *yc - x ;
			cbline(&xl, &ya, &xl, &yb, c);
			xl = *xc - cr;
			cbline(&xl, &ya, &xl, &yb, c);

			if(d + cr > 0)
				d -= (w * --cr) - 1;

			if(x > d)
				d += ((++x) << 1) + 1;
		}

	}
}

/*=====================================================================*/
/**/

void pascal cbdisc(int *xc, int *yc, int *r, int *c)
{
	int w = 4, h = 3;
	cbovaldisc(xc, yc, r, c, &w, &h);
}

/*=====================================================================*/
/**/

void *chrmap(int c)
{
	 char *charbitmap[256] = {
			"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00",
			"\x00\x7E\x81\xA5\x81\x81\xBD\x99\x81\x7E\x00\x00",
			"\x00\x7E\xFF\xDB\xFF\xFF\xC3\xE7\xFF\x7E\x00\x00",
			"\x00\x6C\xFE\xFE\xFE\xFE\x7C\x38\x10\x00\x00\x00",
			"\x00\x00\x10\x38\x7C\xFE\x7C\x38\x10\x00\x00\x00",
			"\x00\x18\x3C\x3C\xE7\xE7\xE7\x99\x18\x3C\x00\x00",
			"\x00\x18\x3C\x7E\xFF\xFF\x7E\x18\x18\x3C\x00\x00",
			"\x00\x00\x00\x18\x3C\x3C\x18\x00\x00\x00\x00\x00",
			"\xFF\xFF\xFF\xFF\xE7\xC3\xC3\xE7\xFF\xFF\xFF\xFF",
			"\x00\x00\x38\x6C\x44\x44\x6C\x38\x00\x00\x00\x00",
			"\x00\xFE\xC6\x92\xBA\xBA\x92\xC6\xFE\x00\x00\x00",
			"\x00\x0E\x06\x0E\x1A\x70\xD8\xD8\x70\x00\x00\x00",
			"\x00\x00\x38\x6C\x6C\x38\x10\x10\x38\x10\x00\x00",
			"\x00\x1C\x36\x36\x3C\x30\x30\x30\xE0\xC0\x00\x00",
			"\x00\x7F\x63\x7F\x63\x63\x67\xE7\xE6\xC0\x00\x00",
			"\x00\x18\x18\xDB\x3C\xE7\x3C\xDB\x18\x18\x00\x00",
			"\x00\x80\xC0\xE0\xF0\xFC\xF0\xE0\xC0\x80\x00\x00",
			"\x00\x02\x06\x0E\x1E\x7E\x1E\x0E\x06\x02\x00\x00",
			"\x00\x18\x3C\x7E\x18\x18\x18\x7E\x3C\x18\x00\x00",
			"\x00\x66\x66\x66\x66\x66\x66\x00\x66\x66\x00\x00",
			"\x00\x7F\xDB\xDB\xDB\x7B\x1B\x1B\x1B\x00\x00\x00",
			"\x7C\xC6\x60\x38\x6C\xC6\x6C\x38\x0C\xC6\x7C\x00",
			"\x00\x00\x00\x00\x00\xFE\xFE\xFE\xFE\x00\x00\x00",
			"\x00\x18\x3C\x7E\x18\x18\x18\x7E\x3C\x18\x7E\x00",
			"\x00\x18\x3C\x7E\x18\x18\x18\x18\x18\x18\x00\x00",
			"\x00\x18\x18\x18\x18\x18\x7E\x3C\x18\x00\x00\x00",
			"\x00\x00\x00\x18\x0C\xFE\x0C\x18\x00\x00\x00\x00",
			"\x00\x00\x00\x30\x60\xFE\x60\x30\x00\x00\x00\x00",
			"\x00\x00\x40\x40\x40\x40\x40\x7E\x00\x00\x00\x00",
			"\x00\x00\x00\x28\x6C\xFE\x6C\x28\x00\x00\x00\x00",
			"\x00\x10\x10\x38\x38\x7C\x7C\xFE\xFE\x00\x00\x00",
			"\x00\xFE\xFE\x7C\x7C\x38\x38\x10\x10\x00\x00\x00",
			"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00",
			"\x00\x18\x3C\x3C\x3C\x18\x18\x00\x18\x18\x00\x00",
			"\x00\x00\x66\x66\x66\x24\x00\x00\x00\x00\x00\x00",
			"\x00\x6C\x6C\xFE\x6C\x6C\xFE\x6C\x6C\x00\x00\x00",
			"\x18\x7C\xC6\xC0\x78\x3C\x06\xC6\x7C\x18\x18\x00",
			"\x00\x00\xC6\xCE\x1C\x38\x70\xE6\xC6\x00\x00\x00",
			"\x38\x6C\x6C\x38\x76\xDC\xCC\xCC\xFC\x76\x00\x00",
			"\x00\x18\x18\x18\x30\x00\x00\x00\x00\x00\x00\x00",
			"\x00\x0C\x18\x30\x30\x30\x30\x30\x18\x0C\x00\x00",
			"\x00\x30\x18\x0C\x0C\x0C\x0C\x0C\x18\x30\x00\x00",
			"\x00\x00\x00\x66\x3C\x7E\x3C\x66\x00\x00\x00\x00",
			"\x00\x00\x00\x18\x18\x7E\x18\x18\x00\x00\x00\x00",
			"\x00\x00\x00\x00\x00\x00\x18\x18\x18\x30\x00\x00",
			"\x00\x00\x00\x00\x00\x00\xFE\x00\x00\x00\x00\x00",
			"\x00\x00\x00\x00\x00\x00\x00\x18\x18\x00\x00\x00",
			"\x00\x00\x02\x06\x0C\x18\x30\x60\xC0\x80\x00\x00",
			"\x00\x7C\xC6\xCE\xD6\xD6\xE6\xC6\x7C\x00\x00\x00",
			"\x00\x18\x38\x18\x18\x18\x18\x18\x3C\x00\x00\x00",
			"\x00\x3C\x66\x0E\x1C\x38\x70\x66\x7E\x00\x00\x00",
			"\x00\x7C\xC6\x06\x1C\x06\x06\xC6\x7C\x00\x00\x00",
			"\x00\x1C\x3C\x6C\xCC\xFE\x0C\x0C\x1E\x00\x00\x00",
			"\x00\xFE\xC0\xFC\xCE\x06\x06\xCE\x7C\x00\x00\x00",
			"\x00\x38\x60\xC0\xFC\xC6\xC6\xC6\x7C\x00\x00\x00",
			"\x00\xFE\xC6\x06\x0C\x18\x30\x30\x30\x00\x00\x00",
			"\x00\x7C\xC6\xC6\x7C\xC6\xC6\xC6\x7C\x00\x00\x00",
			"\x00\x7C\xC6\xC6\xC6\x7E\x06\x06\x0C\x38\x00\x00",
			"\x00\x00\x18\x18\x00\x00\x00\x18\x18\x00\x00\x00",
			"\x00\x00\x18\x18\x00\x00\x00\x18\x18\x30\x00\x00",
			"\x00\x06\x0C\x18\x30\x60\x30\x18\x0C\x06\x00\x00",
			"\x00\x00\x00\x00\x00\xFE\x00\x00\xFE\x00\x00\x00",
			"\x00\x60\x30\x18\x0C\x06\x0C\x18\x30\x60\x00\x00",
			"\x00\x3C\x66\x66\x0C\x18\x18\x00\x18\x18\x00\x00",
			"\x00\x7C\xC6\xC6\xDE\xDE\xDC\xC0\x7C\x00\x00\x00",
			"\x00\x10\x38\x6C\xC6\xFE\xC6\xC6\xC6\x00\x00\x00",
			"\x00\xFC\x66\x66\x7C\x66\x66\x66\xFC\x00\x00\x00",
			"\x00\x7C\xE6\xC6\xC0\xC0\xC6\xE6\x7C\x00\x00\x00",
			"\x00\xFC\x66\x66\x66\x66\x66\x66\xFC\x00\x00\x00",
			"\x00\xFE\x66\x68\x78\x68\x60\x66\xFE\x00\x00\x00",
			"\x00\xFE\x66\x68\x78\x68\x60\x60\xF0\x00\x00\x00",
			"\x00\x7C\xE6\xC0\xC0\xDE\xC6\xE6\x7E\x00\x00\x00",
			"\x00\xC6\xC6\xC6\xFE\xC6\xC6\xC6\xC6\x00\x00\x00",
			"\x00\x3C\x18\x18\x18\x18\x18\x18\x3C\x00\x00\x00",
			"\x00\x1E\x0C\x0C\x0C\x0C\xCC\xCC\x78\x00\x00\x00",
			"\x00\xE6\x66\x6C\x78\x78\x6C\x66\xE6\x00\x00\x00",
			"\x00\xF0\x60\x60\x60\x60\x60\x66\xFE\x00\x00\x00",
			"\x00\xC6\xEE\xFE\xFE\xD6\xC6\xC6\xC6\x00\x00\x00",
			"\x00\xC6\xE6\xF6\xFE\xDE\xCE\xC6\xC6\x00\x00\x00",
			"\x00\x7C\xEE\xC6\xC6\xC6\xC6\xEE\x7C\x00\x00\x00",
			"\x00\xFC\x66\x66\x7C\x60\x60\x60\xE0\x00\x00\x00",
			"\x00\x7C\xC6\xC6\xC6\xC6\xD6\xDE\x7C\x0E\x00\x00",
			"\x00\xFC\x66\x66\x7C\x6C\x66\x66\xE6\x00\x00\x00",
			"\x00\x7C\xC6\xC6\x78\x3C\xC6\xC6\x7C\x00\x00\x00",
			"\x00\x7E\x7E\x5A\x18\x18\x18\x18\x3C\x00\x00\x00",
			"\x00\xC6\xC6\xC6\xC6\xC6\xC6\xC6\x7C\x00\x00\x00",
			"\x00\xC6\xC6\xC6\xC6\xE6\x76\x3C\x18\x00\x00\x00",
			"\x00\xC6\xC6\xC6\xD6\xD6\xFE\x7C\x6C\x00\x00\x00",
			"\x00\xC6\xC6\x6C\x38\x38\x6C\xC6\xC6\x00\x00\x00",
			"\x00\xC6\xC6\xC6\x7C\x38\x30\x18\x7C\x00\x00\x00",
			"\x00\xFE\xC6\x0C\x18\x30\x60\xC6\xFE\x00\x00\x00",
			"\x00\x3C\x30\x30\x30\x30\x30\x30\x30\x3C\x00\x00",
			"\x00\x80\xC0\x60\x30\x18\x0C\x06\x02\x00\x00\x00",
			"\x00\x3C\x0C\x0C\x0C\x0C\x0C\x0C\x0C\x3C\x00\x00",
			"\x10\x38\x6C\xC6\x00\x00\x00\x00\x00\x00\x00\x00",
			"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xFF\x00",
			"\x00\x18\x18\x0C\x00\x00\x00\x00\x00\x00\x00\x00",
			"\x00\x00\x00\x38\x0C\x7C\xCC\xCC\x76\x00\x00\x00",
			"\x00\xE0\x60\x60\x7C\x66\x66\x6E\xDC\x00\x00\x00",
			"\x00\x00\x00\x00\x7C\xC6\xC0\xC6\x7C\x00\x00\x00",
			"\x00\x1C\x0C\x0C\x7C\xCC\xCC\xEC\x76\x00\x00\x00",
			"\x00\x00\x00\x00\x7C\xC6\xFE\xC0\x7C\x00\x00\x00",
			"\x00\x38\x6C\x60\x60\xF8\x60\x60\xF8\x00\x00\x00",
			"\x00\x00\x00\x00\x76\xCC\xCC\xCC\x7C\x0C\x78\x00",
			"\x00\xE0\x60\x60\x6C\x7E\x66\x66\xE6\x00\x00\x00",
			"\x00\x18\x18\x00\x38\x18\x18\x18\x3C\x00\x00\x00",
			"\x00\x0C\x0C\x00\x1C\x0C\x0C\x0C\xCC\xCC\x78\x00",
			"\x00\xE0\x60\x66\x6C\x78\x6C\x66\xE6\x00\x00\x00",
			"\x00\x38\x18\x18\x18\x18\x18\x18\x3C\x00\x00\x00",
			"\x00\x00\x00\x00\xEC\xFE\xD6\xD6\xD6\x00\x00\x00",
			"\x00\x00\x00\x00\xDC\x66\x66\x66\x66\x00\x00\x00",
			"\x00\x00\x00\x00\x3C\x66\x66\x66\x3C\x00\x00\x00",
			"\x00\x00\x00\x00\xDC\x66\x66\x66\x7C\x60\xE0\x00",
			"\x00\x00\x00\x00\x74\xCC\xCC\xCC\x7C\x0C\x1E\x00",
			"\x00\x00\x00\x00\xDC\x76\x62\x60\x60\x00\x00\x00",
			"\x00\x00\x00\x00\x7C\xE0\x7C\x06\x7C\x00\x00\x00",
			"\x00\x10\x30\xFC\x30\x30\x30\x36\x1C\x00\x00\x00",
			"\x00\x00\x00\x00\xCC\xCC\xCC\xCC\x76\x00\x00\x00",
			"\x00\x00\x00\x00\x6C\x6C\x6C\x38\x10\x00\x00\x00",
			"\x00\x00\x00\x00\xC6\xD6\xD6\xFE\x6C\x00\x00\x00",
			"\x00\x00\x00\x00\xC6\x6C\x38\x6C\xC6\x00\x00\x00",
			"\x00\x00\x00\x00\xC6\xC6\xC6\xC6\x7E\x06\x7C\x00",
			"\x00\x00\x00\x00\xFC\x98\x30\x62\xFE\x00\x00\x00",
			"\x00\x0E\x18\x18\x18\x70\x18\x18\x18\x0E\x00\x00",
			"\x00\x18\x18\x18\x18\x00\x18\x18\x18\x18\x00\x00",
			"\x00\x70\x18\x18\x18\x0E\x18\x18\x18\x70\x00\x00",
			"\x00\x76\xDC\x00\x00\x00\x00\x00\x00\x00\x00\x00",
			"\x00\x00\x10\x38\x6C\xC6\xC6\xC6\xFE\x00\x00\x00",
			"\x00\x3C\x66\xC0\xC0\xC2\x66\x3C\x0C\x06\x3C\x00",
			"\x00\xCC\xCC\x00\xCC\xCC\xCC\xCC\x76\x00\x00\x00",
			"\x00\x18\x30\x00\x7C\xC6\xFE\xC0\x7C\x00\x00\x00",
			"\x10\x38\x6C\x00\x38\x0C\x7C\xCC\xCC\x76\x00\x00",
			"\x00\xCC\xCC\x00\x38\x0C\x7C\xCC\xCC\x76\x00\x00",
			"\x00\x30\x18\x00\x78\x0C\x7C\xCC\xCC\x7E\x00\x00",
			"\x38\x6C\x38\x00\x38\x0C\x7C\xCC\xCC\x7E\x00\x00",
			"\x00\x00\x00\x3C\x66\x60\x60\x66\x3C\x0E\x3C\x00",
			"\x10\x38\x6C\x00\x7C\xC6\xFE\xC0\xC6\x7C\x00\x00",
			"\x00\xC6\xC6\x00\x7C\xC6\xFE\xC0\xC6\x7C\x00\x00",
			"\x00\x30\x18\x00\x7C\xC6\xFE\xC0\xC6\x7C\x00\x00",
			"\x00\x66\x66\x00\x38\x18\x18\x18\x18\x3C\x00\x00",
			"\x18\x3C\x66\x00\x38\x18\x18\x18\x18\x3C\x00\x00",
			"\x00\x30\x18\x00\x38\x18\x18\x18\x18\x3C\x00\x00",
			"\x00\xC6\xD6\x38\x6C\xC6\xC6\xFE\xC6\xC6\x00\x00",
			"\x38\x6C\x38\x00\x38\x6C\xC6\xFE\xC6\xC6\x00\x00",
			"\x00\x30\x60\x00\xFE\x66\x78\x68\x66\xFE\x00\x00",
			"\x00\x00\x00\xEC\x36\x36\xD8\xD8\x6E\x00\x00\x00",
			"\x00\x1E\x3C\x6C\xCC\xFE\xCC\xCC\xCC\xCE\x00\x00",
			"\x10\x38\x6C\x00\x7C\xC6\xC6\xC6\xC6\x7C\x00\x00",
			"\x00\xC6\xC6\x00\x7C\xC6\xC6\xC6\xC6\x7C\x00\x00",
			"\x00\x30\x18\x00\x7C\xC6\xC6\xC6\xC6\x7C\x00\x00",
			"\x30\x78\xCC\x00\xCC\xCC\xCC\xCC\xCC\x76\x00\x00",
			"\x00\x18\x0C\x00\xCC\xCC\xCC\xCC\xCC\x76\x00\x00",
			"\x00\xC6\xC6\x00\xC6\xC6\xC6\xC6\x7E\x06\x3C\x00",
			"\x00\xC6\xC6\x38\x6C\xC6\xC6\xC6\x6C\x38\x00\x00",
			"\x00\xC6\xC6\x00\xC6\xC6\xC6\xC6\xC6\x7C\x00\x00",
			"\x00\x18\x18\x3C\x66\x60\x60\x66\x3C\x18\x18\x00",
			"\x00\x38\x6C\x64\xF0\x60\x60\x60\x66\xFC\x00\x00",
			"\x00\x66\x66\x3C\x18\x7E\x18\x7E\x18\x18\x00\x00",
			"\x00\xF8\xCC\xCC\xF8\xC4\xCC\xDE\xCC\xCC\xC6\x00",
			"\x0E\x1B\x18\x18\x18\x7E\x18\x18\x18\xD8\x70\x00",
			"\x00\x18\x30\x00\x38\x0C\x7C\xCC\xCC\x76\x00\x00",
			"\x00\x18\x30\x00\x38\x18\x18\x18\x18\x3C\x00\x00",
			"\x00\x30\x60\x00\x7C\xC6\xC6\xC6\xC6\x7C\x00\x00",
			"\x00\x30\x60\x00\xCC\xCC\xCC\xCC\xCC\x76\x00\x00",
			"\x00\x76\xDC\x00\xDC\x66\x66\x66\x66\x66\x00\x00",
			"\x76\xDC\x00\xC6\xE6\xF6\xFE\xDE\xCE\xC6\xC6\x00",
			"\x00\x3C\x6C\x6C\x3E\x00\x7E\x00\x00\x00\x00\x00",
			"\x00\x38\x6C\x6C\x38\x00\x7C\x00\x00\x00\x00\x00",
			"\x00\x18\x18\x00\x18\x18\x30\x60\x66\x66\x3C\x00",
			"\x00\x00\x00\x00\x00\xFE\xC0\xC0\xC0\x00\x00\x00",
			"\x00\x00\x00\x00\x00\x00\xFE\x06\x06\x00\x00\x00",
			"\x00\xC2\xC6\xCC\xD8\x30\x6E\xD3\x86\x0C\x1F\x00",
			"\x00\xC2\xC6\xCC\xD8\x36\x6E\xDA\xBF\x06\x0F\x00",
			"\x00\x18\x18\x00\x18\x18\x3C\x3C\x3C\x18\x00\x00",
			"\x00\x00\x00\x33\x66\xCC\x66\x33\x00\x00\x00\x00",
			"\x00\x00\x00\xCC\x66\x33\x66\xCC\x00\x00\x00\x00",
			"\x11\x44\x11\x44\x11\x44\x11\x44\x11\x44\x11\x44",
			"\x55\xAA\x55\xAA\x55\xAA\x55\xAA\x55\xAA\x55\xAA",
			"\xDD\x77\xDD\x77\xDD\x77\xDD\x77\xDD\x77\xDD\x77",
			"\x18\x18\x18\x18\x18\x18\x18\x18\x18\x18\x18\x18",
			"\x18\x18\x18\x18\x18\x18\xF8\x18\x18\x18\x18\x18",
			"\x18\x18\x18\x18\xF8\x18\xF8\x18\x18\x18\x18\x18",
			"\x36\x36\x36\x36\x36\x36\xF6\x36\x36\x36\x36\x36",
			"\x00\x00\x00\x00\x00\x00\xFE\x36\x36\x36\x36\x36",
			"\x00\x00\x00\x00\xF8\x18\xF8\x18\x18\x18\x18\x18",
			"\x36\x36\x36\x36\xF6\x06\xF6\x36\x36\x36\x36\x36",
			"\x36\x36\x36\x36\x36\x36\x36\x36\x36\x36\x36\x36",
			"\x00\x00\x00\x00\xFE\x06\xF6\x36\x36\x36\x36\x36",
			"\x36\x36\x36\x36\xF6\x06\xFE\x00\x00\x00\x00\x00",
			"\x36\x36\x36\x36\x36\x36\xFE\x00\x00\x00\x00\x00",
			"\x18\x18\x18\x18\xF8\x18\xF8\x00\x00\x00\x00\x00",
			"\x00\x00\x00\x00\x00\x00\xF8\x18\x18\x18\x18\x18",
			"\x18\x18\x18\x18\x18\x18\x1F\x00\x00\x00\x00\x00",
			"\x18\x18\x18\x18\x18\x18\xFF\x00\x00\x00\x00\x00",
			"\x00\x00\x00\x00\x00\x00\xFF\x18\x18\x18\x18\x18",
			"\x18\x18\x18\x18\x18\x18\x1F\x18\x18\x18\x18\x18",
			"\x00\x00\x00\x00\x00\x00\xFF\x00\x00\x00\x00\x00",
			"\x18\x18\x18\x18\x18\x18\xFF\x18\x18\x18\x18\x18",
			"\x18\x18\x18\x18\x1F\x18\x1F\x18\x18\x18\x18\x18",
			"\x36\x36\x36\x36\x36\x36\x37\x36\x36\x36\x36\x36",
			"\x36\x36\x36\x36\x37\x30\x3F\x00\x00\x00\x00\x00",
			"\x00\x00\x00\x00\x3F\x30\x37\x36\x36\x36\x36\x36",
			"\x36\x36\x36\x36\xF7\x00\xFF\x00\x00\x00\x00\x00",
			"\x00\x00\x00\x00\xFF\x00\xFF\x36\x36\x36\x36\x36",
			"\x36\x36\x36\x36\x37\x30\x37\x36\x36\x36\x36\x36",
			"\x00\x00\x00\x00\xFF\x00\xFF\x00\x00\x00\x00\x00",
			"\x36\x36\x36\x36\xF7\x00\xF7\x36\x36\x36\x36\x36",
			"\x18\x18\x18\x18\xFF\x00\xFF\x00\x00\x00\x00\x00",
			"\x36\x36\x36\x36\xFF\x00\x00\x00\x00\x00\x00\x00",
			"\x00\x00\x00\x00\xFF\x00\xFF\x18\x18\x18\x18\x18",
			"\x00\x00\x00\x00\x00\x00\xFF\x36\x36\x36\x36\x36",
			"\x36\x36\x36\x36\x36\x36\x3F\x00\x00\x00\x00\x00",
			"\x18\x18\x18\x18\x1F\x18\x1F\x00\x00\x00\x00\x00",
			"\x00\x00\x00\x00\x1F\x18\x1F\x18\x18\x18\x18\x18",
			"\x00\x00\x00\x00\x00\x00\x3F\x36\x36\x36\x36\x36",
			"\x36\x36\x36\x36\x36\x36\xFF\x36\x36\x36\x36\x36",
			"\x18\x18\x18\x18\x18\xFF\x18\xFF\x18\x18\x18\x18",
			"\x18\x18\x18\x18\x18\x18\xF8\x00\x00\x00\x00\x00",
			"\x00\x00\x00\x00\x00\x00\x1F\x18\x18\x18\x18\x18",
			"\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF",
			"\x00\x00\x00\x00\x00\x00\xFF\xFF\xFF\xFF\xFF\xFF",
			"\xF0\xF0\xF0\xF0\xF0\xF0\xF0\xF0\xF0\xF0\xF0\xF0",
			"\x0F\x0F\x0F\x0F\x0F\x0F\x0F\x0F\x0F\x0F\x0F\x0F",
			"\xFF\xFF\xFF\xFF\xFF\xFF\x00\x00\x00\x00\x00\x00",
			"\x00\x00\x00\x76\xDC\xD8\xD8\xDC\x76\x00\x00\x00",
			"\x00\x00\x00\xFC\xC6\xFC\xC6\xC6\xFC\xC0\xC0\x00",
			"\x00\xFE\xC6\xC6\xC0\xC0\xC0\xC0\xC0\xC0\x00\x00",
			"\x00\x80\xFE\x6C\x6C\x6C\x6C\x6C\x6C\x00\x00\x00",
			"\x00\xFE\xC6\x60\x30\x18\x30\x60\xC6\xFE\x00\x00",
			"\x00\x00\x00\x7E\xD8\xD8\xD8\xD8\x70\x00\x00\x00",
			"\x00\x00\x00\x66\x66\x66\x66\x66\x7C\x60\xC0\x00",
			"\x00\x00\x76\xDC\x18\x18\x18\x18\x18\x00\x00\x00",
			"\x00\x7E\x18\x3C\x66\x66\x66\x3C\x18\x7E\x00\x00",
			"\x00\x38\x6C\xC6\xC6\xFE\xC6\xC6\x6C\x38\x00\x00",
			"\x00\x38\x6C\xC6\xC6\xC6\x6C\x6C\x6C\xEE\x00\x00",
			"\x00\x00\x1C\x30\x18\x3C\x6C\x6C\x6C\x38\x00\x00",
			"\x00\x00\x00\x7E\xDB\xDB\xDB\x7E\x00\x00\x00\x00",
			"\x00\x00\x03\x06\x7E\xCF\xDB\xF3\x7E\xC0\x00\x00",
			"\x00\x1C\x30\x60\x60\x7C\x60\x60\x30\x1C\x00\x00",
			"\x00\x00\x7C\xC6\xC6\xC6\xC6\xC6\xC6\xC6\x00\x00",
			"\x00\x00\xFE\x00\x00\xFE\x00\x00\xFE\x00\x00\x00",
			"\x00\x00\x18\x18\x7E\x18\x18\x00\xFF\x00\x00\x00",
			"\x00\x30\x18\x0C\x06\x0C\x18\x30\x00\x7E\x00\x00",
			"\x00\x0C\x18\x30\x60\x30\x18\x0C\x00\x7E\x00\x00",
			"\x00\x00\x0E\x1B\x1B\x18\x18\x18\x18\x18\x18\x18",
			"\x18\x18\x18\x18\x18\x18\xD8\xD8\xD8\x70\x00\x00",
			"\x00\x00\x00\x18\x18\x00\x7E\x00\x18\x18\x00\x00",
			"\x00\x00\x00\x76\xDC\x00\x76\xDC\x00\x00\x00\x00",
			"\x00\x38\x6C\x6C\x38\x00\x00\x00\x00\x00\x00\x00",
			"\x00\x00\x00\x00\x00\x18\x18\x00\x00\x00\x00\x00",
			"\x00\x00\x00\x00\x00\x18\x00\x00\x00\x00\x00\x00",
			"\x00\x0F\x0C\x0C\x0C\xEC\x6C\x6C\x3C\x1C\x00\x00",
			"\x00\xD8\x6C\x6C\x6C\x6C\x00\x00\x00\x00\x00\x00",
			"\x00\x70\x98\x30\x60\xC8\xF8\x00\x00\x00\x00\x00",
			"\x00\x00\x00\x7C\x7C\x7C\x7C\x7C\x7C\x00\x00\x00",
			"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
	 };
	 return charbitmap[c];
}

/*=====================================================================*/
/**/

void putdot(int x, int y, int c)
{
	putpixel(&x, &y, &c);
}

/*=====================================================================*/
/**/

void charovergraphic(int chr, int x, int y, int c)
{
	int bottom, byte, top;
	char* map;

	_asm {
		mov dx, 3ceh
		mov al, 04h
		out dx, al
		mov dx, 3c4h
		mov al, 02h
		out dx, al
		mov dx, 3cfh
		mov al, 00h
		out dx, al
		mov dx, 3c5h
		mov al, 01h
		out dx, al
	}

	top = y;
	bottom = top + 12;
	map = chrmap(chr);

	for(byte = 0 ; top < bottom ; top++, byte++)
	{
		if(*(map + byte) & 128)
			fourplanes(x    , top, c, 1);
		if(*(map + byte) &  64)
			fourplanes(x + 1, top, c, 1);
		if(*(map + byte) &  32)
			fourplanes(x + 2, top, c, 1);
		if(*(map + byte) &  16)
			fourplanes(x + 3, top, c, 1);
		if(*(map + byte) &   8)
			fourplanes(x + 4, top, c, 1);
		if(*(map + byte) &   4)
			fourplanes(x + 5, top, c, 1);
		if(*(map + byte) &   2)
			fourplanes(x + 6, top, c, 1);
		if(*(map + byte) &   1)
			fourplanes(x + 7, top, c, 1);
	}

	_asm {
		mov dx, 3cfh
		mov al, 01h
		out dx, al
		mov dx, 3c5h
		mov al, 02h
		out dx, al
	}

	top = y;
	bottom = top + 12;
	map = chrmap(chr);

	for(byte = 0 ; top < bottom ; top++, byte++)
	{
		if(*(map + byte) & 128)
			fourplanes(x    , top, c, 2);
		if(*(map + byte) &  64)
			fourplanes(x + 1, top, c, 2);
		if(*(map + byte) &  32)
			fourplanes(x + 2, top, c, 2);
		if(*(map + byte) &  16)
			fourplanes(x + 3, top, c, 2);
		if(*(map + byte) &   8)
			fourplanes(x + 4, top, c, 2);
		if(*(map + byte) &   4)
			fourplanes(x + 5, top, c, 2);
		if(*(map + byte) &   2)
			fourplanes(x + 6, top, c, 2);
		if(*(map + byte) &   1)
			fourplanes(x + 7, top, c, 2);
	}

	_asm {
		mov dx, 3cfh
		mov al, 02h
		out dx, al
		 mov dx, 3c5h
		mov al, 04h
		out dx, al
	}

	top = y;
	bottom = top + 12;
	map = chrmap(chr);

	for(byte = 0 ; top < bottom ; top++, byte++)
	{
		if(*(map + byte) & 128)
			fourplanes(x    , top, c, 4);
		if(*(map + byte) &  64)
			fourplanes(x + 1, top, c, 4);
		if(*(map + byte) &  32)
			fourplanes(x + 2, top, c, 4);
		if(*(map + byte) &  16)
			fourplanes(x + 3, top, c, 4);
		if(*(map + byte) &   8)
			fourplanes(x + 4, top, c, 4);
		if(*(map + byte) &   4)
			fourplanes(x + 5, top, c, 4);
		if(*(map + byte) &   2)
			fourplanes(x + 6, top, c, 4);
		if(*(map + byte) &   1)
			fourplanes(x + 7, top, c, 4);
	}

	_asm {
		mov dx, 3cfh
		mov al, 03h
		out dx, al
		mov dx, 3c5h
		mov al, 08h
		out dx, al
	}

	top = y;
	bottom = top + 12;
	map = chrmap(chr);

	for(byte = 0 ; top < bottom ; top++, byte++)
	{
		if(*(map + byte) & 128)
			fourplanes(x    , top, c, 8);
		if(*(map + byte) &  64)
			fourplanes(x + 1, top, c, 8);
		if(*(map + byte) &  32)
			fourplanes(x + 2, top, c, 8);
		if(*(map + byte) &  16)
			fourplanes(x + 3, top, c, 8);
		if(*(map + byte) &   8)
			fourplanes(x + 4, top, c, 8);
		if(*(map + byte) &   4)
			fourplanes(x + 5, top, c, 8);
		if(*(map + byte) &   2)
			fourplanes(x + 6, top, c, 8);
		if(*(map + byte) &   1)
			fourplanes(x + 7, top, c, 8);
	}
}

/*=====================================================================*/
/**/

void charout(int chr, int x, int y, int c, int b)
{
	int top, bottom, byte;
	char* map;

	_asm {
		mov dx, 3ceh
		mov al, 04h
		out dx, al
		mov dx, 3c4h
		mov al, 02h
		out dx, al
		mov dx, 3cfh
		mov al, 00h
		out dx, al
		mov dx, 3c5h
		mov al, 01h
		out dx, al
	}

	for(byte = 0, map = chrmap(chr), top = y, bottom = y + 12 ;
		top < bottom ; top++, byte++)
	{
		fourplanes(x    , top, (*(map + byte) & 128) ? c : b, 1);
		fourplanes(x + 1, top, (*(map + byte) &  64) ? c : b, 1);
		fourplanes(x + 2, top, (*(map + byte) &  32) ? c : b, 1);
		fourplanes(x + 3, top, (*(map + byte) &  16) ? c : b, 1);
		fourplanes(x + 4, top, (*(map + byte) &   8) ? c : b, 1);
		fourplanes(x + 5, top, (*(map + byte) &   4) ? c : b, 1);
		fourplanes(x + 6, top, (*(map + byte) &   2) ? c : b, 1);
		fourplanes(x + 7, top, (*(map + byte) &   1) ? c : b, 1);
	}

	_asm {
		mov dx, 3cfh
		mov al, 01h
		out dx, al
		mov dx, 3c5h
		mov al, 02h
		out dx, al
	}

	for(byte = 0, map = chrmap(chr), top = y, bottom = y + 12 ;
		top < bottom ; top++, byte++)
	{
		fourplanes(x    , top, (*(map + byte) & 128) ? c : b, 2);
		fourplanes(x + 1, top, (*(map + byte) &  64) ? c : b, 2);
		fourplanes(x + 2, top, (*(map + byte) &  32) ? c : b, 2);
		fourplanes(x + 3, top, (*(map + byte) &  16) ? c : b, 2);
		fourplanes(x + 4, top, (*(map + byte) &   8) ? c : b, 2);
		fourplanes(x + 5, top, (*(map + byte) &   4) ? c : b, 2);
		fourplanes(x + 6, top, (*(map + byte) &   2) ? c : b, 2);
		fourplanes(x + 7, top, (*(map + byte) &   1) ? c : b, 2);
	}

	_asm {
		mov dx, 3cfh
		mov al, 02h
		out dx, al
		mov dx, 3c5h
		mov al, 04h
		out dx, al
	}

	for(byte = 0, map = chrmap(chr), top = y, bottom = y + 12 ;
		top < bottom ; top++, byte++)
	{
		fourplanes(x    , top, (*(map + byte) & 128) ? c : b, 4);
		fourplanes(x + 1, top, (*(map + byte) &  64) ? c : b, 4);
		fourplanes(x + 2, top, (*(map + byte) &  32) ? c : b, 4);
		fourplanes(x + 3, top, (*(map + byte) &  16) ? c : b, 4);
		fourplanes(x + 4, top, (*(map + byte) &   8) ? c : b, 4);
		fourplanes(x + 5, top, (*(map + byte) &   4) ? c : b, 4);
		fourplanes(x + 6, top, (*(map + byte) &   2) ? c : b, 4);
		fourplanes(x + 7, top, (*(map + byte) &   1) ? c : b, 4);
	}

	_asm {
		mov dx, 3cfh
		mov al, 03h
		out dx, al
		mov dx, 3c5h
		mov al, 08h
		out dx, al
	}

	for(byte = 0, map = chrmap(chr), top = y, bottom = y + 12 ;
		top < bottom ; top++, byte++)
	{
		fourplanes(x    , top, (*(map + byte) & 128) ? c : b, 8);
		fourplanes(x + 1, top, (*(map + byte) &  64) ? c : b, 8);
		fourplanes(x + 2, top, (*(map + byte) &  32) ? c : b, 8);
		fourplanes(x + 3, top, (*(map + byte) &  16) ? c : b, 8);
		fourplanes(x + 4, top, (*(map + byte) &   8) ? c : b, 8);
		fourplanes(x + 5, top, (*(map + byte) &   4) ? c : b, 8);
		fourplanes(x + 6, top, (*(map + byte) &   2) ? c : b, 8);
		fourplanes(x + 7, top, (*(map + byte) &   1) ? c : b, 8);
	}
}

/*=====================================================================*/
/**/

void pascal bitblit(int *left, int *top, int *right, int *bottom, qbstring *pic)
{
	int x, y, byte;
	char* map;

	_asm {
		mov dx, 3ceh
		mov al, 04h
		out dx, al
		mov dx, 3c4h
		mov al, 02h
		out dx, al
		mov dx, 3cfh
		mov al, 00h
		out dx, al
		mov dx, 3c5h
		mov al, 01h
		out dx, al
	}

	for(map = pic->text, y = *top ; y <= *bottom ; y++)
		for(x = *left ; x <= *right ; map++, x++)
			fourplanes(x, y, *map, 1);

	_asm {
		mov dx, 3cfh
		mov al, 01h
		out dx, al
		mov dx, 3c5h
		mov al, 02h
		out dx, al
	}

	for(map = pic->text, y = *top ; y <= *bottom ; y++)
		for(x = *left ; x <= *right ; map++, x++)
			fourplanes(x, y, *map, 2);

	_asm {
		mov dx, 3cfh
		mov al, 02h
		out dx, al
		mov dx, 3c5h
		mov al, 04h
		out dx, al
	}

	for(map = pic->text, y = *top ; y <= *bottom ; y++)
		for(x = *left ; x <= *right ; map++, x++)
			fourplanes(x, y, *map, 4);

	_asm {
		mov dx, 3cfh
		mov al, 03h
		out dx, al
		mov dx, 3c5h
		mov al, 08h
		out dx, al
	}

	for(map = pic->text, y = *top ; y <= *bottom ; y++)
		for(x = *left ; x <= *right ; map++, x++)
			fourplanes(x, y, *map, 8);
}

/*=====================================================================*/
/**/

void pascal transparentblit(int *left, int *top, int *right,
	int *bottom, int *skip, qbstring *pic)
{
	int x, y, byte;
	char *map;

	_asm {
		mov dx, 3ceh
		mov al, 04h
		out dx, al
		mov dx, 3c4h
		mov al, 02h
		out dx, al
		mov dx, 3cfh
		mov al, 00h
		out dx, al
		mov dx, 3c5h
		mov al, 01h
		out dx, al
	}

	for(map = pic->text, y = *top ; y <= *bottom ; y++)
		for(x = *left ; x <= *right ; map++, x++)
			if(*map != (char)*skip)
				fourplanes(x, y, *map, 1);

	_asm {
		mov dx, 3cfh
		mov al, 01h
		out dx, al
		mov dx, 3c5h
		mov al, 02h
		out dx, al
	}

	for(map = pic->text, y = *top ; y <= *bottom ; y++)
		for(x = *left ; x <= *right ; map++, x++)
			if(*map != (char)*skip)
				fourplanes(x, y, *map, 2);

	_asm {
		mov dx, 3cfh
		mov al, 02h
		out dx, al
		mov dx, 3c5h
		mov al, 04h
		out dx, al
	}

	for(map = pic->text, y = *top ; y <= *bottom ; y++)
		for(x = *left ; x <= *right ; map++, x++)
			if(*map != (char)*skip)
				fourplanes(x, y, *map, 4);

	_asm {
		mov dx, 3cfh
		mov al, 03h
		out dx, al
		mov dx, 3c5h
		mov al, 08h
		out dx, al
	}

	for(map = pic->text, y = *top ; y <= *bottom ; y++)
		for(x = *left ; x <= *right ; map++, x++)
			if(*map != (char)*skip)
				fourplanes(x, y, *map, 8);
}

/*=====================================================================*/
/**/

void textout(char *text, int len, int x, int y, int c, int b)
{
	int chr;
	for(chr = 0 ; chr < len ; chr++, x += 8)
		charout(*(text + chr), x, y, c, b);
}

/*=====================================================================*/
/**/

void pascal qbtextout(qbstring *qbs, int *x, int *y, int *c, int *b)
{
	textout(qbs->text, qbs->len, *x, *y, *c, *b);
}

/*=====================================================================*/
/**/

void textovergraphic(char *text, int len, int x, int y, int c)
{
	int chr;
	for(chr = 0; chr < len; chr++, x += 8)
		charovergraphic(*(text + chr), x, y, c);
}

/*=====================================================================*/
/**/

void pascal qbtextovergraphic(qbstring *qbs, int *x, int *y, int *c)
{
	textovergraphic(qbs->text, qbs->len, *x, *y, *c);
}

/*=====================================================================*/
/**/

void pascal mousehide(void)
{
	int iHrz, iVrt, iMv, iMh;
	char *pic = Mm.pic;
	iMv = Mm.v_pos + Md.high;
	iMh = Mm.h_pos + Md.wide;
	for(iVrt = Mm.v_pos ; iVrt < iMv ; iVrt++)
		for(iHrz = Mm.h_pos ; iHrz < iMh ; iHrz++)
			putdot(iHrz, iVrt, *pic++);
}

/*=====================================================================*/
/**/

void pascal mouserestore(void)
{
	int iButton, iHrz, iVrt, iItr, iMv, iMh, pixel;
	char *pic = Md.pic, *bak = Mm.pic;

	ms_set_mouse_pos(64, 64);
	Mm.h_pos = Md.h_pos;
	Mm.v_pos = Md.v_pos;
	iMv = Md.v_pos + Md.high;
	iMh = Md.h_pos + Md.wide;

	for(iVrt = Md.v_pos ; iVrt < iMv ; iVrt++)
		for(iHrz = Md.h_pos ; iHrz < iMh ; iHrz++)
		{
			*bak++ = (char)getdot(iHrz, iVrt);

			if(*pic != ' ')
				putdot(iHrz, iVrt, *pic);

			pic++;
		}
}

/*=====================================================================*/
/**/

void pascal mousedef(int *w, int *h, int *sw, int *sh, qbstring *qbs)
{
	int iItr;
	ms_set_mouse_pos(64, 64);
	Md.wide = *w;
	Md.high = *h;
	Md.sweetleft = *sw;
	Md.sweetdown = *sh;

	for(iItr = 0 ; iItr < 1024 ; iItr++)
			 *(Md.pic + iItr) = *(qbs->text + iItr);
}

/*=====================================================================*/
/**/

void pascal mouseshow(int *h, int *v)
{
	Md.h_pos = *h;
	Md.v_pos = *v;
	mouserestore();
}

/*=====================================================================*/
/**/

int pascal mousemove(int *h, int *v, int *l, int *r)
{
	int iButton, iHrz, iVrt, iItr, iMv, iMh;
	iButton = ms_get_mouse_pos(&iMh, &iMv);

	ms_set_mouse_pos(64, 64);
	Md.l_button = !!(iButton & 1);
	Md.r_button = !!(iButton & 2);
	Md.v_pos += (iMv - 64);
	Md.h_pos += (iMh - 64);

	if(Md.h_pos > 638)
		Md.h_pos = 638;
	if(Md.h_pos < 0)
		Md.h_pos = 0;
	if(Md.v_pos > 478)
		Md.v_pos = 478;
	if(Md.v_pos < 0)
		Md.v_pos = 0;

	while(!(inp(0x3da) & 8));
	while(  inp(0x3da) & 8) ;

	if((Md.h_pos != Mm.h_pos) || (Md.v_pos != Mm.v_pos))
	{
		mousehide();
		mouserestore();
	}

	while(!(inp(0x3da) & 8));
	while(  inp(0x3da) & 8) ;

	*h = Md.h_pos;
	*v = Md.v_pos;
	*l = Md.l_button;
	*r = Md.r_button;

	return iButton;
}

/*=====================================================================*/
/**/

void pascal defaultmouse(void)
{
	qbstring qbs = { 0,
		"hhhhhhhf  "
		"honnnnf   "
		"hononf    "
		"hnonnf    "
		"hoffnnf   "
		"hf  fnnf  "
		"f    fnnf "
		"      fnnf"
		"       fnf"
		"        ff"
	};
	int x = 10, y = 10, r = 0, d = 0;
	mousedef(&x, &y, &r, &d, &qbs);
	x = 120;
	y = 160;
	mouseshow(&x, &y);
}

/*=====================================================================*/
/**/

void button_down(int Btn)
{
	guibutton B = BtnDat[Btn];

	if(B.type == 't')
	{
		framedown(&B.left, &B.top, &B.right, &B.bottom);
	}

	if(B.type == 'm')
	{
		cbblock(&B.left, &B.top, &B.right, &B.bottom, &B.colorfore);
		textovergraphic(B.text, B.len, B.left + 1, B.top, B.colorback);
	}

	if(B.type == 'r')
	{
		int
			trans = 'o', x2 = B.left + 9, y2 = B.top + 9;
		qbstring pic;

		char *rdown = {
			"oooohhoooo"
			"oohhpphhoo"
			"ohppooppho"
			"ohpoppogho"
			"hpoppppogh"
			"hpoppppogh"
			"ohpoppogho"
			"ohggooggho"
			"oohhgghhoo"
			"oooohhoooo"
		};

		pic.text = rdown;
		transparentblit(&B.left, &B.top, &x2, &y2, &trans, &pic);
	}

}

/*=====================================================================*/
/**/

void button_up(int Btn)
{
	guibutton B = BtnDat[Btn];

	if(B.type == 't')
	{
		int iLine, iColumn;
		cbblock(&B.left, &B.top, &B.right, &B.bottom, &B.colorback);
		frameup(&B.left, &B.top, &B.right, &B.bottom);
		iLine = (((B.bottom - B.top) / 2) + B.top) - 5;
		iColumn = (((B.right - B.left) / 2) + B.left) - (B.len * 4);
		textovergraphic(B.text, B.len, iColumn, iLine, B.colorfore);
	}

	if(B.type == 'm')
	{
		cbblock(&B.left, &B.top, &B.right, &B.bottom, &B.colorback);
		textovergraphic(B.text, B.len, B.left + 1, B.top, B.colorfore);
	}

	if(B.type == 'r')
	{
		int trans = 'o', x2 = B.left + 9, y2 = B.top + 9;
		qbstring pic;

		char *rup = {
			"oooohhoooo"
			"oohhpphhoo"
			"ohppooppho"
			"ohpoooogho"
			"hpoooooogh"
			"hpoooooogh"
			"ohpoooogho"
			"ohggooggho"
			"oohhgghhoo"
			"oooohhoooo"
		};

		pic.text = rup;
		cbblock(&B.left, &B.top, &B.right, &B.bottom, &B.colorback);
		transparentblit(&B.left, &B.top, &x2, &y2, &trans, &pic);
		textovergraphic(B.text, B.len, B.left + 12, B.top, B.colorfore);
	}

}

/*=====================================================================*/
/**/

void pascal removebutton(int *b)
{
	if(*b > 0 && *b < 151)
	{
		guibutton B = BtnDat[*b];
		BtnDat[*b] = BtnDat[0];

		if(B.type)
		{
			mousehide();
			cbblock(&B.left, &B.top, &B.right, &B.bottom, &bgcolor);
			mouserestore();
		}
	}
}

/*=====================================================================*/
/**/

void pascal showbutton(int *b)
{
	mousehide();

	if(*b > 0 && *b < 151)
		button_up(*b);

	mouserestore();
}

/*=====================================================================*/
/**/

void pascal showbuttons(int *a, int *z)
{
	int it;
	mousehide();

	for(it = *a ; it <= *z ; it++)
		if(it > 0 && it < 151)
			button_up(it);

	mouserestore();
}

/*=====================================================================*/
/**/

void pascal click(int *b)
{
	guibutton show;
	mousehide();

	if(*b < 0 && *b > -151)
		button_up(-*b);

	if(*b > 0 && *b <  151)
		button_down(*b);

	mouserestore();
}

/*=====================================================================*/
/**/

void pascal regbutton(int *btn, int *type,
	int *l, int *t, int *r, int *b,
	int *cf, int *cb, qbstring *qbs)
{
	int it;

	if(*btn > 0 && *btn < 151)
	{
		BtnDat[*btn].type       = *type;
		BtnDat[*btn].left       = *l;
		BtnDat[*btn].top        = *t;
		BtnDat[*btn].right      = *r;
		BtnDat[*btn].bottom     = *b;
		BtnDat[*btn].colorfore  = *cf;
		BtnDat[*btn].colorback  = *cb;
		BtnDat[*btn].len        = qbs->len;

		for(it = 0 ; it < (qbs->len) && (it < 32) ; it++)
			BtnDat[*btn].text[it] = qbs->text[it];

		if(BtnDat[*btn].len > 32)
			BtnDat[*btn].len = 32;
	}
}

/*=====================================================================*/
/**/

int pascal inbox(int T, int L, int B, int R, int X, int Y)
{
	if(X < L || X > R || Y < T || Y > B)
		return 0;

	return -1;
}

/*=====================================================================*/
/**/

int pascal buttoncheck(int *h, int *v, int *l, int *r)
{
	int iBtn, iMouse, sr, sd, control = 0;
	guibutton B;

	mousemove(h, v, l, r);
	sr = *h + Md.sweetleft;
	sd = *v + Md.sweetdown;

	for(iBtn = 0 ; iBtn < 151 ; iBtn++)
	{
		B = BtnDat[iBtn];

		if(B.type)
		{
			if(inbox(B.top, B.left, B.bottom, B.right, sr, sd))
				control = iBtn;
		}

	}
	return control;
}

/*=====================================================================*/
/**/

void pascal colortheme(qbstring *qbs)
{
	switch(*qbs->text)
	{

		case 'b':
			hil = 9;
			lol = 1;
			break;

		case 'g':
			hil = 10;
			lol =  2;
			break;

		case 'c':
			hil = 11;
			lol =  3;
			break;

		case 'r':
			hil = 13;
			lol =  4;
			break;

		case 'm':
			hil = 13;
			lol =  5;
			break;

		case 'y':
			hil = 14;
			lol =  6;
			break;

		case 'w':
			hil = 15;
			lol =  8;
			break;

	}
}
